from magma_var.magma_var import MagmaVar
from magma_var.utils import transform, extract
from glob import glob
import os
import pandas as pd


class JsonVar(MagmaVar):
    def __init__(self, volcano_code: str, start_date: str, end_date: str,
                 current_dir: str = None, verbose: bool = False):
        super().__init__(volcano_code, start_date, end_date, current_dir, verbose)
        self.files: list[str] = []
        self._df: pd.DataFrame = pd.DataFrame()

    @property
    def df(self) -> pd.DataFrame:
        return self._df

    @df.setter
    def df(self, values: list[dict[str, str]]):
        self._df = pd.DataFrame(values)

    def get(self, from_daily: bool = False):
        _subdir = 'daily' if from_daily is True else 'pages'
        volcano_json_dir = os.path.join(self.json_dir, _subdir, self.volcano_code)

        if self.verbose:
            print(f'ℹ️ Looking JSON files in : {volcano_json_dir}')

        self.files = glob(os.path.join(volcano_json_dir, '*.json'))

        if len(self.files) == 0:
            raise FileNotFoundError(f'⚠️ JSON files not found in {volcano_json_dir}. '
                                    f'Try to download them manually.')

        if self.verbose:
            print(f'ℹ️ Total JSON files :: {len(self.files)}')
            print('=' * 60)

        self.df = extract(events=transform(files=self.files))



colors: dict[str, str] = {
    'Letusan/Erupsi': '#F44336',
    'Awan Panas Letusan': '#e91e63',
    'Awan Panas Guguran': '#673ab7',
    'Guguran': '#1976d2',
    'Hembusan': '#3f51b5',
    'Harmonik': '#607d8b',
    'Tremor Non-Harmonik': '#0d47a1',
    'Tornillo': '#03a9f4',
    'Low Frequency': '#006064',
    'Hybrid/Fase Banyak': '#009688',
    'Vulkanik Dangkal': '#8BC34A',
    'Vulkanik Dalam': '#33691E',
    'Very Long Period': '#827717',
    'Tektonik Lokal': '#F57F17',
    'Terasa': '#FFCA28',
    'Tektonik Jauh': '#FFA726',
    'Double Event': '#ff5722',
    'Getaran Banjir': '#795548',
    'Deep Tremor': '#333333',
    'Tremor Menerus': '#9E9E9E',
}

columns: dict[str, str] = {
    'gempa.letusan_erupsi': 'Letusan/Erupsi',
    'gempa.awan_panas_letusan': 'Awan Panas Letusan',
    'gempa.awan_panas_guguran': 'Awan Panas Guguran',
    'gempa.guguran': 'Guguran',
    'gempa.hembusan': 'Hembusan',
    'gempa.harmonik': 'Harmonik',
    'gempa.tremor_non-_harmonik': 'Tremor Non-Harmonik',
    'gempa.tornillo': 'Tornillo',
    'gempa.low_frequency': 'Low Frequency',
    'gempa.hybrid_fase_banyak': 'Hybrid/Fase Banyak',
    'gempa.vulkanik_dangkal': 'Vulkanik Dangkal',
    'gempa.vulkanik_dalam': 'Vulkanik Dalam',
    'gempa.very_long_period': 'Very Long Period',
    'gempa.tektonik_lokal': 'Tektonik Lokal',
    'gempa.terasa': 'Terasa',
    'gempa.tektonik_jauh': 'Tektonik Jauh',
    'gempa.double_event': 'Double Event',
    'gempa.getaran_banjir': 'Getaran Banjir',
    'gempa.deep_tremor': 'Deep Tremor',
    'gempa.tremor_menerus': 'Tremor Menerus'
}

event_codes = ['*', 'lts', 'apl', 'apg', 'gug', 'hbs', 'hrm', 'tre',
               'tor', 'lof', 'hyb', 'vtb', 'vta', 'vlp', 'tel', 'trs',
               'tej', 'dev', 'gtb', 'dpt', 'mtr']

event_names: dict[str, str] = {
    'lts': 'Letusan/Erupsi',
    'apl': 'Awan Panas Letusan',
    'apg': 'Awan Panas Guguran',
    'gug': 'Guguran',
    'hbs': 'Hembusan',
    'hrm': 'Harmonik',
    'tre': 'Tremor Non-Harmonik',
    'tor': 'Tornillo',
    'lof': 'Low Frequency',
    'hyb': 'Hybrid/Fase Banyak',
    'vtb': 'Vulkanik Dangkal',
    'vta': 'Vulkanik Dalam',
    'vlp': 'Very Long Period',
    'tel': 'Tektonik Lokal',
    'trs': 'Terasa',
    'tej': 'Tektonik Jauh',
    'dev': 'Double Event',
    'gtb': 'Getaran Banjir',
    'dpt': 'Deep Tremor',
    'mtr': 'Tremor Menerus'
}



from magma_var.resources import event_codes, colors, columns
from magma_var.utils import check_directory
from typing import Tuple
from datetime import datetime, timedelta
from pandas.errors import EmptyDataError
from pathlib import Path
import os
import json
import pandas as pd
import requests
import matplotlib.pyplot as plt
import matplotlib.dates as mdates


def validate_earthquake_events(earthquake_events: str | list[str] = None) -> list[str]:
    if earthquake_events is None:
        return ['*']

    if isinstance(earthquake_events, str):
        earthquake_events = [earthquake_events]

    for earthquake_event in earthquake_events:
        if earthquake_event not in event_codes:
            raise ValueError("Earthquake_events must be one of '*', 'lts', 'apl', 'apg', 'gug', 'hbs', 'hrm', "
                             "'tre', 'tor', 'lof', 'hyb', 'vtb', 'vta','vlp', 'tel', 'trs', 'tej', 'dev', 'gtb', "
                             "'dpt', 'mtr'")

    return earthquake_events


class Plot:
    url_evaluasi = 'https://magma.esdm.go.id/api/v1/python/magma-var/evaluasi'

    def __init__(self, token: str, volcano_code: str = None, start_date: str = None, end_date: str = None,
                 earthquake_events: str | list[str] = None, current_dir: str = None, verbose: bool = False) -> None:

        self.token = token
        self.volcano_code = volcano_code
        self.start_date = start_date
        self.end_date = end_date
        self.earthquake_events: list[str] = validate_earthquake_events(earthquake_events)
        self.verbose = verbose
        self.figsize = (12,1)
        self.figure_ylabel_x = 0.07
        self.figure_ylabel_fontsize = 12
        self.title_y_location = 0.92
        self.title_fontszie = 12
        self.legend_fontsize = 8
        self.x_labelsize = 8
        self.y_labelsize = 8

        self.json: dict = self.get_json_response()
        self.df: pd.DataFrame = self.get_df()
        self.events_not_recorded = self.df.columns[self.df.sum() == 0]
        self.filename = 'magma_{}_{}_{}'.format(self.volcano_code, self.start_date, self.end_date)

        if current_dir is None:
            current_dir = os.getcwd()
        self.current_dir = current_dir

        self.output_dir, self.figures_dir, self.magma_dir = check_directory(current_dir)

        if start_date is None:
            self.start_date: str = datetime.today().strftime('%Y-%m-%d')

        if end_date is None:
            self.end_date = (datetime.today() + timedelta(days=30)).strftime('%Y-%m-%d')

        self.csv = self.download()

    def get_df(self, json_response: dict = None) -> pd.DataFrame:
        if json_response is None:
            json_response = self.json

        df = pd.json_normalize(json_response['data'])
        df.drop(columns=[
            'availability',
            'visual.visibility',
            'visual.cuaca',
            'visual.asap.teramati',
            'visual.asap.warna',
            'visual.asap.intensitas',
            'visual.asap.tekanan',
            'visual.asap.tinggi_min',
            'visual.asap.tinggi_max',
            'visual.letusan.teramati',
            'visual.letusan.tinggi_min',
            'visual.letusan.tinggi_max',
            'visual.letusan.warna',
            'visual.awan_panas_guguran.teramati',
            'visual.awan_panas_guguran.jarak_min',
            'visual.awan_panas_guguran.jarak_max'
        ], inplace=True)

        df.drop(columns=df.columns[df.sum() == 0], inplace=True)
        df.set_index(keys='date', inplace=True)
        df.index = pd.to_datetime(df.index)

        df.rename(columns=columns, inplace=True)

        if 'Tremor Menerus' in df.columns:
            df.drop(columns=['Tremor Menerus'], inplace=True)

        return df

    def get_json_response(self, token: str = None) -> dict:

        if token is None:
            token = self.token

        url = self.url_evaluasi

        payload = json.dumps({
            "start_date": self.start_date,
            "end_date": self.end_date,
            "code_ga": self.volcano_code,
            "gempa": self.earthquake_events
        })

        headers = {
            'Authorization': 'Bearer ' + token,
            'Content-Type': 'application/json'
        }

        start_date_object = datetime.strptime(self.start_date, "%Y-%m-%d")
        end_date_object = datetime.strptime(self.end_date, "%Y-%m-%d")

        if start_date_object > end_date_object:
            raise ValueError(
                'End date ({}) must be greater than start date ({})'.format(self.end_date, self.start_date))

        if (end_date_object > datetime.now()) or (start_date_object > datetime.now()):
            raise ValueError('End date or start date must not greater than today ({})'.format(datetime.today().date()))

        try:
            response = requests.request("GET", url, headers=headers, data=payload).json()
        except Exception as e:
            raise ValueError(f'Please check your token or parameters {payload}. Error: {e}')

        if 'code' in response:
            if response['code'] == 401:
                raise ValueError(f'Please update your token at https://magma.esdm.go.id/chambers/token')

        if 'errors' in response:
            raise ValueError(response['errors'])

        if self.verbose:
            print(response)

        return response

    def download(self) -> str:
        """Download daily seismic from MAGMA Indonesia

        Returns:
            str: path to downloaded file
        """
        if not self.df.empty:
            try:
                csv = os.path.join(self.magma_dir, "{}.csv".format(self.filename))
                self.df.to_csv(csv)
                self.csv = csv
                print(f'💾 Saved to {csv}')

                return csv
            except Exception as e:
                raise FileNotFoundError(f'Failed to save csv file: {e}')
        else:
            print('⚠️ There is no event(s) between {} and {}. '
                  'Please change your parameters.'.format(self.start_date, self.end_date))
            raise EmptyDataError

    def from_csv(self, csv: str, interval: int = 1, width: float = 0.5, save_plot: bool = True,
                 title: str = None, dpi: int = 300) -> Tuple[plt.Figure, plt.Axes]:
        """Plot from csv file.

        Args:
            csv (str): path to csv file
            interval (int, optional): Xtick label interval (day). Defaults to 1.
            width (float, optional): Width of column bar. Defaults to 0.5.
            save_plot (bool, optional): Save plot. Defaults to True.
            title (str, optional): Title of plot. Defaults to 'Kegempaan'.
            dpi (int, optional): Image resolution. Defaults to 300.

        Returns:
            Tuple[plt.Figure, plt.Axes]: Figure, Axes
        """

        if title is None:
            title = 'Kegempaan'

        figure_width, figure_height = self.figsize

        filename = Path(csv).stem

        df = pd.read_csv(csv, index_col='date', parse_dates=True)

        fig, axs = plt.subplots(
            nrows=len(df.columns),
            ncols=1,
            figsize=(figure_width, figure_height * len(df.columns)),
            sharex=True
        )

        plt.subplots_adjust(hspace=0.0)

        for gempa, column_name in enumerate(df.columns):
            Plot.ax(
                ax=axs[gempa],
                df=df,
                column_name=column_name,
                width=width,
                interval=interval,
                legend_fontsize=self.legend_fontsize,
                x_labelsize=self.x_labelsize,
                y_labelsize=self.y_labelsize
            )

        fig.supylabel('Jumlah', x=self.figure_ylabel_x, fontsize=self.figure_ylabel_fontsize)
        fig.suptitle(title, fontsize=self.title_fontszie, y=self.title_y_location)

        if save_plot:
            figure_name = os.path.join(self.figures_dir, f'{filename}.png')
            print(f'📷 Saved to : {figure_name}')
            fig.savefig(figure_name, dpi=dpi)

        return fig, axs

    def show(self, interval: int = 1, width: float = 0.5, save_plot: bool = True,
             title: str = None, dpi: int = 300, figsize: Tuple[int, int] = (12, 1),
             title_y_location: float = 0.92, title_fontsize: int = 12,
             figure_ylabel_x: float = 0.07, figure_ylabel_fontsize: int = 12,
             legend_fontsize: int = 8, x_labelsize: int = 8, y_labelsize: int = 8) -> Tuple[plt.Figure, plt.Axes]:
        """Show plot in matplotlib

        Args:
            interval (int, optional): Xtick label interval (day). Defaults to 1.
            width (float, optional): Width of column bar. Defaults to 0.5.
            save_plot (bool, optional): Save plot. Defaults to True.
            title (str, optional): Title of plot. Defaults to 'Kegempaan'.
            dpi (int, optional): Image resolution. Defaults to 300.
            figsize (tuple(int, int), optional): Size of figure. Defaults to (12, 1).
            title_y_location (float, optional): Location of title. Defaults to 0.92.
            title_fontsize (int, optional): Font size of title. Defaults to 12.
            figure_ylabel_x (float, optional): Location x of figure. Defaults to 0.07.
            figure_ylabel_fontsize (int, optional): Font size of figure. Defaults to 12.
            legend_fontsize (int, optional): Font size of legend. Defaults to 8.
            x_labelsize (int, optional): Font size of x-axis label. Defaults to 8.
            y_labelsize (int, optional): Font size of y-axis label. Defaults to 8.

        Returns:
            Tuple[plt.Figure, plt.Axes]: Figure, Axes
        """
        self.figsize = figsize
        self.title_y_location = title_y_location
        self.title_fontszie = title_fontsize
        self.figure_ylabel_fontsize = figure_ylabel_fontsize
        self.figure_ylabel_x = figure_ylabel_x
        self.legend_fontsize = legend_fontsize

        self.x_labelsize = x_labelsize
        self.y_labelsize = y_labelsize

        return self.from_csv(
            csv=self.csv,
            interval=interval,
            width=width,
            save_plot=save_plot,
            title=title,
            dpi=dpi
        )

    @staticmethod
    def ax(ax: plt.Axes, df: pd.DataFrame, column_name: str, width: float = 0.5, interval=1,
           legend_fontsize = 8, x_labelsize = 8, y_labelsize = 8) -> plt.Axes:

        ax.bar(df.index, df[column_name], width=width, label=column_name,
               color=colors[column_name], linewidth=0)

        ax.legend(loc=2, fontsize=legend_fontsize)

        ax.xaxis.set_major_locator(mdates.DayLocator(interval=interval))
        ax.xaxis.set_major_formatter(mdates.DateFormatter('%Y-%m-%d'))

        ax.yaxis.get_major_ticks()[0].label1.set_visible(False)
        ax.set_xlim(df.first_valid_index(), df.last_valid_index())

        ax.set_ylim([0, df[column_name].max() * 1.2])

        for label in ax.get_xticklabels(which='major'):
            label.set(rotation=30, horizontalalignment='right')

        ax.tick_params(axis='y', labelsize=x_labelsize)
        ax.tick_params(axis='x', labelsize=y_labelsize)

        # for key, continuous in enumerate(continuous_eruptions):
        #     # continuous[0] = start date of eruption
        #     # continuous[1] = end date of eruption
        #     axs[gempa].axvspan(continuous[0], continuous[1], alpha=0.4,
        #                        color='orange', label="_" * key + 'Continuous Eruption')
        #
        # for key, date in enumerate(single_eruptions):
        #     axs[gempa].axvline(datetime.strptime(date, '%Y-%m-%d'),
        #                        color='red', label="_" * key + 'Single Eruption')

        return ax



from typing import Tuple
import os
import json
import re


def check_directory(current_dir: str) -> Tuple[str, str, str]:
    """Checking existing directory

    Args:
        current_dir (str): Current directory

    Returns:
        Tuple[str, str, str]: Directory name, directory path, and file path
    """
    output_dir = os.path.join(current_dir, 'output')
    os.makedirs(output_dir, exist_ok=True)

    figures_dir = os.path.join(output_dir, 'figures')
    os.makedirs(figures_dir, exist_ok=True)

    magma_dir = os.path.join(output_dir, 'magma')
    os.makedirs(magma_dir, exist_ok=True)

    return output_dir, figures_dir, magma_dir


def transform(files: list[str], index: int = 0) -> list[dict[str, str]]:
    """Transform all json files

    Args:
        files (list[str]): list of json files
        index (int, optional): Starting index. Defaults to 0.

    Returns:
        dict[str, str]: transformed json files
    """
    events: list[dict[str, str]] = []
    for file in files:
        response = json.load(open(file))
        data = response['data'] if 'data' in response.keys() else response
        for _data in data:
            notice_number: str = _data['laporan_terakhir']['noticenumber']
            year = notice_number[0:4]
            month = notice_number[4:6]
            day = notice_number[6:8]
            descriptions: list[str] = _data['laporan_terakhir']['gempa']['deskripsi']
            for description in descriptions:
                index = index + 1
                _events: dict[str, str] = {
                    'index': int(index),
                    'date': f'{year}-{month}-{day}',
                    'event': description
                }
                events.append(_events)

    if len(events) > 0:
        events.sort(key=lambda x: x['index'], reverse=True)

    return events


def extracted(date: str, description: str, event_name: str = None, count: int = None,
              amplitude: str = None, sp_time: str = None, duration: str = None,
              dominant: str = None) -> dict[str, str]:
    amp_min = None
    amp_max = amplitude
    sp_min = None
    sp_max = sp_time
    duration_min = None
    duration_max = duration

    if amplitude is not None:
        amps = amplitude.split('-')
        if len(amps) == 2:
            amp_min, amp_max = amplitude.split('-')

    if sp_time is not None:
        sp_times = sp_time.split('-')
        if len(sp_times) == 2:
            sp_min, sp_max = sp_time.split('-')

    if duration is not None:
        durations = duration.split('-')
        if len(durations) == 2:
            duration_min, duration_max = durations

    return {
        'date': date,
        'description': description,
        'event_name': event_name,
        'count': count,
        'amplitude_min': float(amp_min) if amp_min is not None else None,
        'amplitude_max': float(amp_max) if amp_max is not None else None,
        'sp_min': float(sp_min) if sp_min is not None else None,
        'sp_max': float(sp_max) if sp_max is not None else None,
        'duration_min': float(duration_min) if duration_min is not None else None,
        'duration_max': float(duration_max) if duration_max is not None else None,
        'dominant': float(dominant) if dominant is not None else None,
    }


def extract(events: list[dict[str, str]]) -> list[dict[str, str]]:
    extracted_list: list[dict[str, str]] = []

    for event in events:
        date = event['date']
        text: str = event['event'].strip()

        name_match_pattern = r'gempa\s+(.+?)\s+dengan'
        if 'Harmonik' in text:
            name_match_pattern = r'kali\s+(.+?)\s+dengan'

        count_match = re.search(r'(\d+)\s+kali', text)
        name_match = re.search(name_match_pattern, text)
        amp_match = re.search(r'amplitude\s+([\d\-.]+)\s+mm', text)
        sp_match = re.search(r'S-P\s+([\d\-.]+)\s+detik', text)
        duration_match = re.search(r'lama\s+gempa\s+([\d\-.]+)\s+detik', text)
        dominant_match = re.search(r'dominan\s+([\d\-.]+)\s+mm', text)

        try:
            if count_match and name_match and amp_match:
                data = {
                    'count': int(count_match.group(1)),
                    'event_name': name_match.group(1).strip(),
                    'amplitude': amp_match.group(1),
                    'sp': sp_match.group(1) if sp_match else None,
                    'duration': duration_match.group(1) if duration_match else None,
                    'dominant': dominant_match.group(1) if dominant_match else None
                }

                extracted_list.append(extracted(
                    date=date,
                    description=text,
                    event_name=data['event_name'],
                    count=int(data['count']),
                    amplitude=data['amplitude'],
                    sp_time=data['sp'],
                    duration=data['duration'],
                    dominant=data['dominant']
                ))
            else:
                extracted_list.append(extracted(
                    date=date,
                    description=text,
                ))
        except Exception as e:
            print(f'❌ Cannot extract of date {date} :: {text}')
            print(e)

    return extracted_list


def save(filename: str, response: dict) -> str:
    with open(filename, 'w', encoding='utf-8') as f:
        json.dump(response, f, ensure_ascii=False, indent=4)

    return filename



from magma_var.utils import save, transform as tf, extract as ext
from magma_var import MagmaVar
import os
import json
import pandas as pd
import requests
from typing import Self


class Download(MagmaVar):
    url_filter = 'https://magma.esdm.go.id/api/v1/magma-var/filter'
    url_volcano = 'https://magma.esdm.go.id/api/v1/home/gunung-api'

    def __init__(self, token: str, volcano_code: str, start_date: str, end_date: str,
                 current_dir: str = None, verbose: bool = False, url_filter: str = None, url_volcano: str = None):
        super().__init__(volcano_code, start_date, end_date, current_dir, verbose)

        self.token = token
        self.url_filter = url_filter or self.url_filter
        self.url_volcano = url_volcano or self.url_volcano

        self.page_dir = os.path.join(self.json_dir, 'pages', self.volcano_code)
        self.daily_dir = os.path.join(self.json_dir, 'daily', self.volcano_code)

        self.json_filename = f"{self.filename}"
        self.files: list[str] = []
        self.events: list[dict[str, str]] = []
        self._df: pd.DataFrame = pd.DataFrame()

        self.headers = {
            'Authorization': 'Bearer ' + self.token,
            'Content-Type': 'application/json'
        }

        if self.verbose:
            print(f'ℹ️ JSON Directory: {self.json_dir}')
            print(f'ℹ️ Volcano Directory: {self.page_dir}')

    @property
    def df(self) -> pd.DataFrame:
        return self._df

    @df.setter
    def df(self, values: list[dict[str, str]]):
        self._df = pd.DataFrame(values)

    def _download_daily(self, data: dict, overwrite: bool = False) -> None:
        """Saving daily json file

        Args:
            data (dict): Data from response['data']
            overwrite (bool, optional): Overwrite existing file. Defaults to False.

        Returns:
            None
        """
        notice_number: str = data['laporan_terakhir']['noticenumber']
        year = notice_number[0:4]
        month = notice_number[4:6]
        day = notice_number[6:8]
        filename = f'{year}-{month}-{day}.json'
        filepath = os.path.join(self.daily_dir, filename)

        if os.path.isfile(filepath) and not overwrite:
            if self.verbose:
                print(f'ℹ️ File Exists. Skipping: {filepath}')
            return None

        save(filepath, data)

        if self.verbose:
            print(f'🗃️ File Saved: {filepath}')

        return None

    def download_daily(self, overwrite: bool = False) -> None:
        """Download as daily json file

        Args:
            overwrite (bool, optional): Overwrite file if exists. Defaults to False.
        """
        os.makedirs(self.daily_dir, exist_ok=True)

        if self.verbose:
            print('=' * 60)
            print(f'ℹ️ Downloading first page')

        response = self.download()
        first_page_data = response['data']

        if self.verbose:
            print(f'ℹ️ Data found : {len(first_page_data)}')
            print('=' * 60)

        for _data in first_page_data:
            self._download_daily(_data, overwrite)

        # Downloading rest of data
        last_page = response['last_page']
        if last_page == 1:
            pass

        if self.verbose:
            print('=' * 60)
            print(f'ℹ️ Downloading from page 2 to {last_page}')
            print('=' * 60)

        for page in range(2, last_page + 1):
            response = self.download({'page': page})
            for _data in response['data']:
                self._download_daily(_data, overwrite)

    def download(self, params: dict = None) -> dict:
        """Download VAR from Filter URL

        Args:
            params (dict): Page parameter to pass

        Returns:
            response (dict): Dictionary of response
        """
        volcano_code = self.volcano_code
        start_date = self.start_date
        end_date = self.end_date

        payload: str = json.dumps({
            "code": volcano_code,
            "start_date": start_date,
            "end_date": end_date,
        })

        try:
            response = requests.request("GET", self.url_filter, headers=self.headers,
                                        data=payload, params=params).json()

            if 'errors' in response.keys():
                raise ValueError(f'❌ Download Error :: {response["errors"]}')

            if 'data' not in response.keys():
                raise KeyError(f'❌ Data not found in response :: {response}')

            if len(response['data']) == 0:
                raise ValueError(f'❌ No data found for {self.volcano_code} :: {response}')

            return response
        except Exception as e:
            raise ValueError(f'❌ Failed to download JSON :: {e}')

    def download_per_page(self, response: dict) -> list[str]:
        """Download VAR per page from Filter URL

        Args:
            response (dict): Dictionary of response

        Returns:
            list of filenames
        """
        pages: list[str] = []
        last_page = response['last_page']

        if last_page == 1:
            pass

        if self.verbose:
            print(f'ℹ️ Downloading from page 2 to {last_page}')
            print('=' * 60)

        for page in range(2, last_page + 1):
            # code_startDate_endDate_page.json
            # example: AWU_2025-01-01_2025-01-31_1.json
            filename = f'{self.json_filename}_{page}.json'
            json_per_page = os.path.join(self.page_dir, filename)

            if os.path.exists(json_per_page):
                if self.verbose:
                    print(f'✅ Skip. JSON for page #{page} exists :: {json_per_page}')
                pages.append(json_per_page)
                continue

            response = self.download({'page': page})
            save(json_per_page, response)

            pages.append(json_per_page)

            if self.verbose:
                print(f'✅ JSON for page #{page} downloaded :: {json_per_page}')
                print('=' * 60)

        self.files = self.files + pages
        return pages

    def download_first_page(self) -> dict:
        """Download first page to be used as reference and get metadata pagination

        Returns:
            response (dict): Dictionary of response
        """
        first_page_json = os.path.join(self.page_dir, f'{self.json_filename}_1.json')

        if os.path.isfile(first_page_json):
            if self.verbose:
                print(f'✅ JSON First Page exists :: {first_page_json}')
            self.files.append(first_page_json)
            return json.load(open(first_page_json))

        if self.verbose:
            print(f'⌛ Downloading JSON First Page :: {first_page_json}')

        response = self.download()

        save(first_page_json, response)

        if self.verbose:
            total = response['total']
            print(f'ℹ️ Total Data :: {total}')
            print(f'✅ JSON First Page downloaded :: {first_page_json}')

        self.files.append(first_page_json)

        return response

    def extract(self) -> Self:
        """Extract description

        Returns:
            self: Self
        """
        self.df = ext(events=self.events)
        return self

    def transform(self) -> Self:
        """Transform all json files

        Returns:
            self: Self
        """
        self.events = tf(files=self.files)
        return self

    def var(self, as_daily: bool = False) -> Self:
        """Download VAR from Filter URL and save it to JSON files

        Args:
            as_daily (bool, optional): Whether to save daily json file. Defaults to False.

        Returns:
            self: Self
        """
        if as_daily:
            if self.verbose:
                print(f'ℹ️ Save VAR as daily JSON file.')
            self.download_daily(overwrite=True)
            return self

        os.makedirs(self.page_dir, exist_ok=True)

        response = self.download_first_page()
        self.download_per_page(response)

        if self.verbose:
            print('ℹ️ JSON Files :: ', len(self.files))

        self.transform().extract()

        if len(self.events) != len(self.df):
            if self.verbose:
                print(f'⚠️ Events length not equal to DataFrame length :: {len(self.events)} vs {len(self.df)}')
            print(f'⚠️ Please kindly to check the results.')

        return self

    def _to(self, filetype: str, filename: str = None, verbose: bool = True) -> str | None:
        """Save wrapper

        Args:
            filetype (str): Type of wrapper. 'csv' or 'excel'
            filename (str, optional): Name of file. Defaults to None.
            verbose (bool, optional): Print verbose output. Defaults to True.

        Returns:
            Saved path location
        """
        if len(self.df) == 0:
            print(f'⚠️ No data found.')
            return None

        to_dir = os.path.join(self.magma_dir, filetype)
        os.makedirs(to_dir, exist_ok=True)

        _filename = filename if filename is not None else self.filename
        path = os.path.join(to_dir, f'{_filename}.{filetype}')

        self.df.to_csv(path, index=False) if filetype == 'csv' else self.df.to_excel(path, index=False)

        if verbose:
            print(f'🗃️ Saved to :: {path}')

        return path

    def to_csv(self, filename: str = None, verbose: bool = True) -> str | None:
        """Save to CSV files

        Args:
            filename (str, optional): CSV filename. Defaults to None.
            verbose (bool, optional): For debugging. Defaults to True.

        Returns:
            str: CSV file location
        """
        return self._to(filetype='csv', filename=filename, verbose=verbose)

    def to_excel(self, filename: str = None, verbose: bool = True) -> str | None:
        """Save to Excel Files

        Args:
            filename (str, optional): Excel filename. Defaults to None.
            verbose (bool, optional): For debugging. Defaults to True.
        """
        return self._to(filetype='xlsx', filename=filename, verbose=verbose)



from magma_var.utils import check_directory
import os


class MagmaVar:
    def __init__(self, volcano_code: str,
                 start_date: str, end_date: str, current_dir: str = None,
                 verbose: bool = False):
        self.volcano_code = volcano_code
        self.start_date = start_date
        self.end_date = end_date

        if current_dir is None:
            current_dir = os.getcwd()
        self.current_dir = current_dir

        self.output_dir, self.figures_dir, self.magma_dir = check_directory(current_dir)

        self.json_dir = os.path.join(self.magma_dir, 'json')
        os.makedirs(self.json_dir, exist_ok=True)

        self.filename = f"{self.volcano_code}_{self.start_date}_{self.end_date}"

        self.verbose = verbose
